<?php
namespace Teknisa\PLA\Modules\Relatorio\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;


class Pla06300PesquisaDeConsumoDeProdutos extends \Zeedhi\Framework\Controller\Report {

	protected $crudService;
	protected $reportService;
	protected $newCode;
	protected $service;
	protected $qr2Strategy;
	protected $birtStrategy;
	protected $environment;

	public function __construct($reportService, $crudService, $newCode, $service, $qr2Strategy, $birtStrategy, $environment){
		$this->crudService = $crudService;
		$this->reportService = $reportService;
		$this->newCode = $newCode;
		$this->service = $service;
		$this->qr2Strategy = $qr2Strategy;
		$this->birtStrategy = $birtStrategy;
		$this->environment = $environment;
		parent::__construct($reportService);
	}

	protected $reportMapping = array (
  		'PLA06300' => array (
			'strategy' => 'BIRT',
			'parameterMapping' =>
			array (
				"P_DTINICIAL" => "P_DTINICIAL",
				"P_DTFINAL" => "P_DTFINAL",
				"P_CDRECINICIALSEL" => "P_CDRECINICIALSEL",
				"P_CDRECFINALSEL" => "P_CDRECFINALSEL",
				"P_ORDENA" =>  "P_ORDENA",
				"P_CDPRATINI" => "P_CDPRATINI",
				"P_CDPRATFIN" => "P_CDPRATFIN",
				"P_NRLISTAPROD" => "P_NRLISTAPROD",
				"P_NRCONSPLAN" => "P_NRCONSPLAN",
				"P_CARDAPIO" => "P_CARDAPIO",
				"P_FIXO" => "P_FIXO",
				"P_PERCAPITA" => "P_PERCAPITA",
				"NRORG"     => "P_NRORG"
			)
		)
	);
	public function findFilgrupfili(DTO\Request\Filter $request, DTO\Response $response) {

		try {
			$filterCriteria = $request->getFilterCriteria();

			$filterCriteria->setDataSourceName('pla06300_filial_filgrupfili_filioper');
	    $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
	    $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array('CDOPERADOR','P_CDGRUPFILI','NRORG'));
	    $this->crudService->find($response, $newFilterCriteria);
		} catch (\Exception $e) {
			$response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
		}
	}

	public function findServicoFiliais(DTO\Request\Filter $request, DTO\Response $response){
		try {
			$filterCriteria = $request->getFilterCriteria();

			$filterCriteria->setDataSourceName('pla06300_servico_filiais');
		    $filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		    $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, array('P_CDFILIAL','NRORG'));
		    $this->crudService->find($response, $newFilterCriteria);
		} catch (\Exception $e) {
			$response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
		}
	}

	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
		$row['P_CDRECFINALSEL'] = $row['P_CDPRATFIN'];
		$row['P_CDRECINICIALSEL'] = $row['P_CDPRATINI'];

		$row['P_CDPRATFIN'] = str_pad($row['P_CDPRATFIN'], 10, "z", STR_PAD_RIGHT);
		$this->qr2Strategy->setQr2Type($row['QR2TYPE']);
		$this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));

		$row['P_NRLISTAPROD'] = $this->newCode->getCodeByTableName('LISTA', null, 1, 10, false);
		$this->service->insereLista($row); //INCLUIR LISTA COM SERVIÇOS E FILIAIS
		$row['P_NRCONSPLAN'] = $this->service->callConsplan($row)['P_NRCONSPLAN'];
		$row['P_NRLISTAPROD'] = $this->newCode->getCodeByTableName('LISTA', null, 1, 10, false);
		$this->service->includeListaProd($row); // INLCUIR LISTA COM PRODUTOS
	}
}