<?php
namespace Teknisa\PLA\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

use Teknisa\Libs\Util\SMTP;

class UtilSMTP {
	
	private $conn;
	private $env;
	private $pathToSave = '/tmp/';
	private $temporaryAttachments = [];
	private $smtpParams = [];
	private $subject = '';

	public function __construct($connection, $environment, $smtpParams = []){
		$this->conn = $connection;
		$this->env = $environment;
		if(empty($smtpParams)){
			$this->smtpParams = InstanceManager::getInstance()->getParameter('smtp_params');
		} else {
			$this->smtpParams = $smtpParams;
		}
	}

	public function findSmtpParamsUnit( $cdfilial ){
		$sql = <<<SQL
			SELECT
				NVL(DSLOGINSMTP, DSEMAILFILI) AS USERNAME,
				DSSMTPEMAILF   AS HOST,
				CDSENHEMAILF   AS PASSWORD,
				CASE
					WHEN IDUTTLSEMAIL = 'S' THEN
						'tls'
					WHEN IDUTSSLEMAIL = 'S' THEN
						'ssl'
					ELSE
						''
				END AS SMTPSECURE,
				NVL(DSEMAILFILI, DSLOGINSMTP) AS EMAIL_FROM,
				CASE
					WHEN NVL(DSLOGINSMTP, DSEMAILFILI) IS NOT NULL
						AND CDSENHEMAILF IS NOT NULL THEN
						'S'
					ELSE
						'N'
				END AS AUTH
			FROM
				ENDEFILI
			WHERE
				CDFILIAL = :P_CDFILIAL
				AND IDTPENDEFILI = 'P'
				AND DSSMTPEMAILF IS NOT NULL
SQL;
		$stmt = $this->conn->prepare($sql);
		$stmt->execute([
			'P_CDFILIAL' => $cdfilial
		]);
		$result = $stmt->fetch();
		$params = [];
		if(!empty($result)){
			$params = [
				'host'		 => $result['HOST'],
				'username'	 => $result['USERNAME'],
				'password'	 => $result['PASSWORD'],
				'smtpSecure' => $result['SMTPSECURE'],
				'from'		 => $result['EMAIL_FROM'],
				'smtpAuth' 	 => $result['AUTH']
			];
			$hostAndPort = self::getSmtpSettingsFromAddress($params['host']);
			$params['host'] = $hostAndPort['host'];
			$params['port'] = $hostAndPort['port'] ?? '';
			$params['password'] = self::getDelphiPasswordEncrypted($params['password'], false);
		}
		return $params;
	}
	
	public static function getSmtpSettingsFromAddress($url)
    {
        $match = self::splitByCommonDelimiters($url);
        
        if (empty($match)) {
            throw new \Exception('INVALID_SMTP_HOST');
        }
        
        if (count($match) > 1) {
            $params['port'] = $match[1];
        }
        
        $params['host'] = $match[0];
        
        return $params;
    }
        
    private static function splitByCommonDelimiters($string) {
        $pattern = '/[:;|]+/';
        $cleanedString = self::removeAllWhitespaces($string);
        $splittedString = preg_split($pattern, $cleanedString);
        return $splittedString;
    }
    
    private static function removeAllWhitespaces($string) {
        $pattern = '/\s+/';
        $cleanedString = preg_replace($pattern, '', $string);
        return $cleanedString;
    }
    
     public static function getDelphiPasswordEncrypted($password, $isEncrypt = true, $offset = '2')
    {
        $url =  self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );
    
        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($offset, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );
    
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUncripto';
        
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );
    
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
    
        return $encryption;
    }
    
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
    
    public function findEmailUnit(DTO\Request\Row $request, DTO\Response $response){
		$sql = <<<SQL
		SELECT
			DSEMAILFILI AS EMAIL_FROM
		FROM
			ENDEFILI
		WHERE
			( CDFILIAL = :P_CDFILIAL )
			AND IDTPENDEFILI = 'P'
			AND NRORG = :P_NRORG
SQL;
		$row = $request->getRow();
		$cdfilial = $row['CDFILIAL'] ?? null;
		$stmt = $this->conn->prepare($sql);
		$stmt->execute([
			'P_CDFILIAL' => $cdfilial,
			'P_NRORG'    => $this->env->getNrOrg()
		]);
		$result = $stmt->fetch();
		if($result === false) $result = ["EMAIL_FROM" => null];
		$response->addDataSet( new DataSet("result", $result) );
	}

}